@@ -20,7 +20,6 @@
 #include "weatherpopupapplet.h"
 
 #include <QTimer>
-#include <QIcon>
 
 #include <KConfigGroup>
 #include <KLocalizedString>
@@ -28,22 +27,19 @@
 
 #include <KUnitConversion/Converter>
 
-#include <Plasma/PluginLoader>
-#include <plasma_version.h>
-
 #include "weatherlocation.h"
 
 using namespace KUnitConversion;
 
-class WeatherPopupApplet::Private
+class Q_DECL_HIDDEN WeatherPopupApplet::Private
 {
 public:
     Private(WeatherPopupApplet *weatherapplet)
         : q(weatherapplet)
-        , weatherEngine(nullptr)
-        , timeEngine(nullptr)
         , updateInterval(0)
         , location(nullptr)
+        , latitude(qQNaN())
+        , longitude(qQNaN())
         , timeoutNotification(nullptr)
     {
         busyTimer = new QTimer(q);
@@ -53,8 +49,6 @@ public:
     }
 
     WeatherPopupApplet *q;
-    Plasma::DataEngine *weatherEngine;
-    Plasma::DataEngine *timeEngine;
     Converter converter;
     Unit temperatureUnit;
     Unit speedUnit;
@@ -73,6 +67,13 @@ public:
     QTimer *busyTimer;
     KNotification *timeoutNotification;
 
+    bool isValidLatitude() {
+        return -90 <= latitude && latitude <= 90;
+    }
+    bool isValidLongitude() {
+        return -180 <= longitude && longitude <= 180;
+    }
+
     void locationReady(const QString &src)
     {
         if (!src.isEmpty()) {
@@ -87,10 +88,7 @@ public:
             if (timeoutNotification) {
                 timeoutNotification->close();
             }
-            QObject *graphicObject = q->property("_plasma_graphicObject").value<QObject *>();
-            if (graphicObject) {
-                graphicObject->setProperty("busy", false);
-            }
+            q->setBusy(false);
             q->setConfigurationRequired(true);
         }
 
@@ -100,10 +98,7 @@ public:
 
     void giveUpBeingBusy()
     {
-        QObject *graphicObject = q->property("_plasma_graphicObject").value<QObject *>();
-        if (graphicObject) {
-            graphicObject->setProperty("busy", false);
-        }
+        q->setBusy(false);
 
         QStringList list = source.split(QLatin1Char( '|' ), QString::SkipEmptyParts);
         if (list.count() < 3) {
@@ -112,7 +107,7 @@ public:
             timeoutNotification =
                 KNotification::event( KNotification::Error, QString(), // TODO: some title?
                                       i18n("Weather information retrieval for %1 timed out.", list.value(2)),
-                                      QLatin1String("dialog-error"));
+                                      QStringLiteral("dialog-error"));
             QObject::connect(timeoutNotification, SIGNAL(closed()), q, SLOT(onTimeoutNotificationClosed()));
         }
     }
@@ -140,7 +135,7 @@ public:
     {
         QString result;
         if (!pressure.isValid()) {
-            return QLatin1String( "weather-none-available" );
+            return QStringLiteral("weather-none-available");
         }
         qreal temp = temperature.convertTo(Celsius).number();
         qreal p = pressure.convertTo(Kilopascal).number();
@@ -150,50 +145,48 @@ public:
         p += t * 10;
 
         // PORT!
-//         if (!timeEngine) {
-//             timeEngine = Plasma::PluginLoader::self()->loadDataEngine( QStringLiteral("time") );
-//         }
+//         Plasma::DataEngine *timeEngine = dataEngine(QStringLiteral("time"));
 //         Plasma::DataEngine::Data data = timeEngine->query(
 //                 QString(QLatin1String( "Local|Solar|Latitude=%1|Longitude=%2" )).arg(latitude).arg(longitude));
         bool day = true;//(data[QLatin1String( "Corrected Elevation" )].toDouble() > 0.0);
 
         if (p > 103.0) {
             if (day) {
-                result = QLatin1String( "weather-clear" );
+                result = QStringLiteral("weather-clear");
             } else {
-                result = QLatin1String( "weather-clear-night" );
+                result = QStringLiteral("weather-clear-night");
             }
         } else if (p > 100.0) {
             if (day) {
-                result = QLatin1String( "weather-clouds" );
+                result = QStringLiteral("weather-clouds");
             } else {
-                result = QLatin1String( "weather-clouds-night" );
+                result = QStringLiteral("weather-clouds-night");
             }
         } else if (p > 99.0) {
             if (day) {
                 if (temp > 1.0) {
-                    result = QLatin1String( "weather-showers-scattered-day" );
+                    result = QStringLiteral("weather-showers-scattered-day");
                 } else if (temp < -1.0)  {
-                    result = QLatin1String( "weather-snow-scattered-day" );
+                    result = QStringLiteral("weather-snow-scattered-day");
                 } else {
-                    result = QLatin1String( "weather-snow-rain" );
+                    result = QStringLiteral("weather-snow-rain");
                 }
             } else {
                 if (temp > 1.0) {
-                    result = QLatin1String( "weather-showers-scattered-night" );
+                    result = QStringLiteral("weather-showers-scattered-night");
                 } else if (temp < -1.0)  {
-                    result = QLatin1String( "weather-snow-scattered-night" );
+                    result = QStringLiteral("weather-snow-scattered-night");
                 } else {
-                    result = QLatin1String( "weather-snow-rain" );
+                    result = QStringLiteral("weather-snow-rain");
                 }
             }
         } else {
             if (temp > 1.0) {
-                result = QLatin1String( "weather-showers" );
+                result = QStringLiteral("weather-showers");
             } else if (temp < -1.0)  {
-                result = QLatin1String( "weather-snow" );
+                result = QStringLiteral("weather-snow");
             } else {
-                result = QLatin1String( "weather-snow-rain" );
+                result = QStringLiteral("weather-snow-rain");
             }
         }
         //kDebug() << result;
@@ -224,13 +217,6 @@ WeatherPopupApplet::~WeatherPopupApplet()
 
 void WeatherPopupApplet::init()
 {
-    // workaround for weather dataengine being destructed behind our back when there are no
-    // current connections for a while
-    // get the weather dataengine and do a dummy connect for ions source, never to be disconnected
-    // this will keep the refcount of connections for the dataengine always above 0
-    d->weatherEngine = Plasma::PluginLoader::self()->loadDataEngine( QStringLiteral("weather") );
-    d->weatherEngine->connectSource(QStringLiteral("ions"), this);
-
     configChanged();
 }
 
@@ -251,15 +237,11 @@ void WeatherPopupApplet::connectToEngine()
             connect(d->location, SIGNAL(finished(QString)), this, SLOT(locationReady(QString)));
         }
 
-        Plasma::DataEngine *dataEngine =
-            Plasma::PluginLoader::self()->loadDataEngine( QStringLiteral("geolocation") );
+        Plasma::DataEngine *dataEngine = dataEngine(QStringLiteral("geolocation"));
         d->location->setDataEngines(dataEngine, d->weatherEngine);
         d->location->getDefault();
 #endif
-        QObject *graphicObject = property("_plasma_graphicObject").value<QObject *>();
-        if (graphicObject) {
-            graphicObject->setProperty("busy", false);
-        }
+        setBusy(false);
         // TODO: remove once WeatherLocation is fixed above
         // fake d->location returning no result
         d->locationReady(QString());
@@ -267,13 +249,11 @@ void WeatherPopupApplet::connectToEngine()
         delete d->location;
         d->location = nullptr;
 
-        QObject *graphicObject = property("_plasma_graphicObject").value<QObject *>();
-        if (graphicObject) {
-            graphicObject->setProperty("busy", true);
-        }
+        setBusy(true);
         d->busyTimer->start();
 
-        d->weatherEngine->connectSource(d->source, this, d->updateInterval * 60 * 1000);
+        Plasma::DataEngine* weatherDataEngine = dataEngine(QStringLiteral("weather"));
+        weatherDataEngine->connectSource(d->source, this, d->updateInterval * 60 * 1000);
     }
 }
 
@@ -282,25 +262,25 @@ void WeatherPopupApplet::saveConfig(const QVariantMap& configChanges)
     KConfigGroup cfg = config();
 
     // units
-    if (configChanges.contains("temperatureUnitId")) {
-        cfg.writeEntry("temperatureUnit", configChanges.value("temperatureUnitId").toInt());
+    if (configChanges.contains(QStringLiteral("temperatureUnitId"))) {
+        cfg.writeEntry("temperatureUnit", configChanges.value(QStringLiteral("temperatureUnitId")).toInt());
     }
-    if (configChanges.contains("windSpeedUnitId")) {
-        cfg.writeEntry("speedUnit", configChanges.value("windSpeedUnitId").toInt());
+    if (configChanges.contains(QStringLiteral("windSpeedUnitId"))) {
+        cfg.writeEntry("speedUnit", configChanges.value(QStringLiteral("windSpeedUnitId")).toInt());
     }
-    if (configChanges.contains("pressureUnitId")) {
-        cfg.writeEntry("pressureUnit", configChanges.value("pressureUnitId").toInt());
+    if (configChanges.contains(QStringLiteral("pressureUnitId"))) {
+        cfg.writeEntry("pressureUnit", configChanges.value(QStringLiteral("pressureUnitId")).toInt());
     }
-    if (configChanges.contains("visibilityUnitId")) {
-        cfg.writeEntry("visibilityUnit", configChanges.value("visibilityUnitId").toInt());
+    if (configChanges.contains(QStringLiteral("visibilityUnitId"))) {
+        cfg.writeEntry("visibilityUnit", configChanges.value(QStringLiteral("visibilityUnitId")).toInt());
     }
 
     // data source
-    if (configChanges.contains("updateInterval")) {
-        cfg.writeEntry("updateInterval", configChanges.value("updateInterval").toInt());
+    if (configChanges.contains(QStringLiteral("updateInterval"))) {
+        cfg.writeEntry("updateInterval", configChanges.value(QStringLiteral("updateInterval")).toInt());
     }
-    if (configChanges.contains("source")) {
-        cfg.writeEntry("source", configChanges.value("source").toString());
+    if (configChanges.contains(QStringLiteral("source"))) {
+        cfg.writeEntry("source", configChanges.value(QStringLiteral("source")).toString());
     }
 
     emit configNeedsSaving();
@@ -309,7 +289,8 @@ void WeatherPopupApplet::saveConfig(const QVariantMap& configChanges)
 void WeatherPopupApplet::configChanged()
 {
     if (!d->source.isEmpty()) {
-        d->weatherEngine->disconnectSource(d->source, this);
+        Plasma::DataEngine* weatherDataEngine = dataEngine(QStringLiteral("weather"));
+        weatherDataEngine->disconnectSource(d->source, this);
     }
 
     KConfigGroup cfg = config();
@@ -332,80 +313,73 @@ void WeatherPopupApplet::configChanged()
     connectToEngine();
 }
 
+inline double readGeoDouble(const Plasma::DataEngine::Data &data, const QString &key)
+{
+    const Plasma::DataEngine::Data::ConstIterator it = data.find(key);
+
+    if (it == data.end()) {
+        return qQNaN();
+    }
+
+    bool ok = false;
+    double result = it.value().toDouble(&ok);
+    if (!ok) {
+        result = qQNaN();
+    }
+    return result;
+}
+
 void WeatherPopupApplet::dataUpdated(const QString& source,
                                      const Plasma::DataEngine::Data &data)
 {
-    if (source == QLatin1String("ions")) {
-        return;
-    }
+    Q_UNUSED(source);
 
     if (data.isEmpty()) {
         return;
     }
 
-    d->conditionIcon = data[QLatin1String( "Condition Icon" )].toString();
+    d->conditionIcon = data[QStringLiteral("Condition Icon")].toString();
     if (data[QLatin1String( "Pressure" )].toString() != QLatin1String( "N/A" )) {
-        d->pressure = Value(data[QLatin1String( "Pressure" )].toDouble(),
-                            static_cast<UnitId>(data[QLatin1String( "Pressure Unit" )].toInt()));
+        d->pressure = Value(data[QStringLiteral("Pressure")].toDouble(),
+                            static_cast<UnitId>(data[QStringLiteral("Pressure Unit")].toInt()));
     } else {
         d->pressure = Value();
     }
-    d->tend = data[QLatin1String( "Pressure Tendency" )].toString();
-    d->temperature = Value(data[QLatin1String( "Temperature" )].toDouble(),
-                           static_cast<UnitId>(data[QLatin1String( "Temperature Unit" )].toInt()));
-    d->latitude = data[QLatin1String( "Latitude" )].toDouble();
-    d->longitude = data[QLatin1String( "Longitude" )].toDouble();
+    d->tend = data[QStringLiteral("Pressure Tendency")].toString();
+    d->temperature = Value(data[QStringLiteral("Temperature")].toDouble(),
+                           static_cast<UnitId>(data[QStringLiteral("Temperature Unit")].toInt()));
+    d->latitude = readGeoDouble(data, QStringLiteral("Latitude"));
+    d->longitude = readGeoDouble(data, QStringLiteral("Longitude"));
     const QString creditUrl = data[QStringLiteral("Credit Url")].toString();
-#if PLASMA_VERSION >= 0x052200
     QList<QUrl> associatedApplicationUrls;
     if (!creditUrl.isEmpty()) {
         associatedApplicationUrls << creditUrl;
     }
     setAssociatedApplicationUrls(associatedApplicationUrls);
-#else
-    // older plasma framework versions are exposed to a crash risk if
-    // the associatedApplicationUrls are explicitely set to an empty list.
-    // So once some urls had been set before and we need to update them to the current,
-    // always use a non-empty list.
-    // As we do not have a proper url as fallback if there is no credit url given,
-    // just set an empty url (which creditUrl then is, so same code works).
-    // That empty url will result in the associated-application action being active and
-    // activating it will result in a "bogus url" error. Which seems the least grave error
-    // case to escape into here.
-    const QList<QUrl> oldAssociatedApplicationUrls = associatedApplicationUrls();
-    if (!creditUrl.isEmpty() || !oldAssociatedApplicationUrls.isEmpty()) {
-        QList<QUrl> associatedApplicationUrls;
-        associatedApplicationUrls << creditUrl;
-        setAssociatedApplicationUrls(associatedApplicationUrls);
-    }
-#endif
 
     d->busyTimer->stop();
     if (d->timeoutNotification) {
         d->timeoutNotification->close();
     }
-    QObject *graphicObject = property("_plasma_graphicObject").value<QObject *>();
-    if (graphicObject) {
-        graphicObject->setProperty("busy", false);
-    }
+    setBusy(false);
 }
 
-Unit WeatherPopupApplet::pressureUnit()
+Unit WeatherPopupApplet::pressureUnit() const
 {
     return d->pressureUnit;
 }
 
-Unit WeatherPopupApplet::temperatureUnit()
+Unit WeatherPopupApplet::temperatureUnit() const
 {
     return d->temperatureUnit;
 }
 
-Unit WeatherPopupApplet::speedUnit()
+Unit WeatherPopupApplet::speedUnit() const
 {
     return d->speedUnit;
 }
 
-Unit WeatherPopupApplet::visibilityUnit()
+Unit WeatherPopupApplet::visibilityUnit() const
 {
     return d->visibilityUnit;
 }
@@ -421,14 +395,14 @@ QVariantMap WeatherPopupApplet::configValues() const
     QVariantMap config;
 
     // units
-    config.insert("temperatureUnitId", d->temperatureUnit.id());
-    config.insert("windSpeedUnitId", d->speedUnit.id());
-    config.insert("pressureUnitId", d->pressureUnit.id());
-    config.insert("visibilityUnitId", d->visibilityUnit.id());
+    config.insert(QStringLiteral("temperatureUnitId"), d->temperatureUnit.id());
+    config.insert(QStringLiteral("windSpeedUnitId"), d->speedUnit.id());
+    config.insert(QStringLiteral("pressureUnitId"), d->pressureUnit.id());
+    config.insert(QStringLiteral("visibilityUnitId"), d->visibilityUnit.id());
 
     // data source
-    config.insert("updateInterval", d->updateInterval);
-    config.insert("source", d->source);
+    config.insert(QStringLiteral("updateInterval"), d->updateInterval);
+    config.insert(QStringLiteral("source"), d->source);
 
     return config;
 }
